package com.mortals.xhx.module.attendance.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.aliyun.dingtalkworkflow_1_0.models.ListProcessInstanceIdsResponseBody;
import com.google.common.collect.Lists;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.OrderCol;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.util.DataUtil;
import com.mortals.framework.util.DateUtils;
import com.mortals.xhx.base.system.user.service.UserService;
import com.mortals.xhx.common.code.*;
import com.mortals.xhx.module.attendance.dao.AttendanceLeaveRecordDao;
import com.mortals.xhx.module.attendance.model.AttendanceLeaveRecordEntity;
import com.mortals.xhx.module.attendance.model.AttendanceLeaveRecordQuery;
import com.mortals.xhx.module.attendance.model.AttendanceRecordHikEntity;
import com.mortals.xhx.module.attendance.model.vo.AttendanceSummaryQuery;
import com.mortals.xhx.module.attendance.service.AttendanceGroupStaffService;
import com.mortals.xhx.module.attendance.service.AttendanceLeaveRecordService;
import com.mortals.xhx.module.attendance.service.AttendanceRecordHikService;
import com.mortals.xhx.module.attendance.service.AttendanceStatService;
import com.mortals.xhx.module.dingding.personal.service.IDingPersonService;
import com.mortals.xhx.module.staff.model.StaffEntity;
import com.mortals.xhx.module.staff.service.StaffService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import static com.mortals.xhx.common.key.RedisKey.KEY_ATTENDANCE_STAT_CACHE;

/**
 * AttendanceLeaveRecordService
 * 请假记录信息 service实现
 *
 * @author zxfei
 * @date 2023-04-07
 */
@Service("attendanceLeaveRecordService")
@Slf4j
public class AttendanceLeaveRecordServiceImpl extends AbstractCRUDServiceImpl<AttendanceLeaveRecordDao, AttendanceLeaveRecordEntity, Long> implements AttendanceLeaveRecordService {

    @Autowired
    private AttendanceStatService attendanceStatService;
    @Autowired
    private ICacheService cacheService;
    @Autowired
    private StaffService staffService;
    @Autowired
    private UserService userService;
    @Autowired
    private AttendanceRecordHikService hikService;
    @Autowired
    private IDingPersonService dingPersonService;
    @Autowired
    private AttendanceGroupStaffService attendanceGroupStaffService;

    /**
     * 员工事假请假数据key 员工id+开始时间+结束时间
     */
    private static String LEAVE_REIDS_KEY = "attendance:leave:";

    @Override
    public AttendanceLeaveRecordEntity update(AttendanceLeaveRecordEntity entity, Context context) throws AppException {
        this.updateBefore(entity, context);
        int iRet = this.dao.update(entity);
        if (iRet == 0) {
            throw new AppException(-1002, "更新失败！");
        } else {
            super.updateAfter(entity, context);
            doUpdateRecordAndSummary(entity, context);
            return entity;
        }
    }

    @Override
    public AttendanceLeaveRecordEntity doUpdateRecord(String processInstanceId, AttendanceLeaveRecordEntity leaveRecordEntity) throws Exception {
        if (leaveRecordEntity.getAuditResult() != AppealResultEnum.审核中.getValue()) {
            //临时解决并发问题，更新的都等待一秒后再执行
            Thread.sleep(1000);
        }
        synchronized (this) {
            AttendanceLeaveRecordEntity recordEntity = this.selectOne(new AttendanceLeaveRecordQuery().remark(processInstanceId));
            if (ObjectUtils.isEmpty(recordEntity)) {
                leaveRecordEntity.setCreateTime(new Date());
                leaveRecordEntity.setCreateUserId(1L);
                this.save(leaveRecordEntity);
            } else {
                leaveRecordEntity.setId(recordEntity.getId());
                leaveRecordEntity.setUpdateUserId(1L);
                leaveRecordEntity.setUpdateTime(new Date());
                this.update(leaveRecordEntity);
            }
            return leaveRecordEntity;
        }
    }


    @Override
    protected void saveAfter(AttendanceLeaveRecordEntity entity, Context context) throws AppException {
        super.saveAfter(entity, context);
        doUpdateRecordAndSummary(entity, context);
    }

    /**
     * 更新汇总信息
     *
     * @param entity
     * @param context
     */
    private void doUpdateRecordAndSummary(AttendanceLeaveRecordEntity entity, Context context) {
        //请假是未来 不做处理
        int compare = DateUtil.compare(entity.getStartTime(), new Date());
        if (compare > 0) {
            //请假未来 不做处理
            log.info("请假未来 不做处理! {}", entity.getLeavePerson());
            return;
        }
        //更新考勤记录与异常记录
        if (AppealResultEnum.通过.getValue() == entity.getAuditResult() || AppealResultEnum.撤销审批通过.getValue() == entity.getAuditResult()) {
            AttendanceRecordHikEntity hikEntity = new AttendanceRecordHikEntity();
            hikEntity.setStaffId(entity.getLeavePersonId());
            //判断请假的开始日期与结束日期
  /*          boolean in = DateUtil.isIn(new Date(), entity.getStartTime(), entity.getEndTime());
            if (in) {
                //当前日期在请假期间，计算考勤结束时间为当前时间;
                attendanceRecordHikQuery.setAttendanceDateStart(DateUtil.formatDate(entity.getStartTime()));
                attendanceRecordHikQuery.setAttendanceDateEnd(DateUtil.today());
            } else {
                attendanceRecordHikQuery.setAttendanceDateStart(DateUtil.formatDate(entity.getStartTime()));
                attendanceRecordHikQuery.setAttendanceDateEnd(DateUtil.formatDate(entity.getEndTime()));
            }*/

            DateTime attendStart = DateUtil.parseDate(DateUtil.formatDate(entity.getStartTime()));
            DateTime attendEnd = DateUtil.parseDate(DateUtil.formatDate(entity.getEndTime()));
            Long compare1 = DateUtil.betweenDay(attendEnd, attendStart, true);
            log.info("考勤计算天数区间：{}", compare1);
            for (int i = 0; i <= compare1.intValue(); i++) {
                DateTime curDate = DateUtil.offsetDay(attendStart, i);
                hikEntity.setAttendanceDateStart(curDate.toDateStr());
                hikEntity.setAttendanceDateEnd(curDate.toDateStr());
                hikEntity.setOrderColList(Arrays.asList(new OrderCol("attendanceDate")));
                try {
                    hikService.addAttendanceRecordByQuery(hikEntity, context);
                } catch (Exception e) {
                    log.error("同步请假记录异常", e);

                }
            }
            log.info("请假通过后汇总考勤，请假人员：" + entity.getLeavePerson());
            AttendanceSummaryQuery query = new AttendanceSummaryQuery();
            query.setSummaryTimeEnd(DateUtils.getStrDate(entity.getEndTime()));
            query.setSummaryTimeStart(DateUtils.getStrDate(entity.getStartTime()));
            query.setStaffId(entity.getLeavePersonId());
            cacheService.lpush(KEY_ATTENDANCE_STAT_CACHE, query);
      /*      AddAttendThread addAttendThread = new AddAttendThread(hikService, attendanceRecordHikQuery, context);
            ThreadPool.getInstance().execute(addAttendThread);*/
        }
    }


    @Override
    public Rest<String> syncLeaveRecord(AttendanceLeaveRecordQuery query, Context context) {
        List<AttendanceLeaveRecordEntity> waitSyncLeaveRecords = new ArrayList<>();
        Long staffId = query.getStaffId();
        List<StaffEntity> staffCacheList = staffService.getCacheList();
        // List<UserEntity> userCacheList = userService.getCacheList();
        if (ObjectUtils.isEmpty(staffId)) {
            //同步所有用户
            List<StaffEntity> userEntityList = staffCacheList.stream()
                    .filter(f -> !ObjectUtils.isEmpty(f.getDingUserId())).collect(Collectors.toList());
            getLeaveRecordByUserList(query, userEntityList, waitSyncLeaveRecords);
        } else {
            //同步指定用户
            List<StaffEntity> userEntityList = staffCacheList.stream()
                    .filter(f -> !ObjectUtils.isEmpty(f.getDingUserId()))
                    .collect(Collectors.toList());
            getLeaveRecordByUserList(query, userEntityList, waitSyncLeaveRecords);
        }
        log.info("待同步请假记录：{}", JSON.toJSONString(waitSyncLeaveRecords));
        if (!ObjectUtils.isEmpty(waitSyncLeaveRecords)) {
            for (AttendanceLeaveRecordEntity waitSyncLeaveRecord : waitSyncLeaveRecords) {
                dingPersonService.handleByProcessInstanceId(waitSyncLeaveRecord.getRemark());
            }
        }
        return Rest.ok();
    }

    @Value("${source.type:''}")
    protected String sourceType;

    private void getLeaveRecordByUserList(AttendanceLeaveRecordQuery query, List<StaffEntity> staffCacheList, List<AttendanceLeaveRecordEntity> waitSyncLeaveRecords) {
        List<List<StaffEntity>> partition = Lists.partition(staffCacheList, 10);
        for (List<StaffEntity> staffList : partition) {
            //String dingUserId = item.getDingUserId();
            List<String> dingUserIds = staffList.stream().map(m -> m.getDingUserId()).collect(Collectors.toList());
            long startTime = DateUtil.parseDateTime(query.getStartTimeStart() + " 00:00:00").getTime();
            long endTime = DateUtil.parseDateTime(query.getEndTimeEnd() + " 23:59:59").getTime();
            Long nextToken = 0L;
            long maxResults = 20L;
            if (AttendSourceTypeEnum.海康云平台.getValue().equals(sourceType)) {

                for (ProcessInstanceEnum processInstanceEnum : ProcessInstanceEnum.values()) {
                    try {
                        getRecords(waitSyncLeaveRecords, processInstanceEnum, startTime, endTime, nextToken, maxResults, dingUserIds);
                    } catch (Exception e) {
                        log.error("同步请假记录异常", e);
                    }
                }
            }  else if (AttendSourceTypeEnum.钉钉.getValue().equals(sourceType)) {
                for (QiongLaiProcessInstanceEnum processInstanceEnum : QiongLaiProcessInstanceEnum.values()) {
                    try {
                        getRecords(waitSyncLeaveRecords, processInstanceEnum, startTime, endTime, nextToken, maxResults, dingUserIds);
                    } catch (Exception e) {
                        log.error("同步请假记录异常", e);
                    }
                }
            }else {
                for (ProcessInstanceEnum processInstanceEnum : ProcessInstanceEnum.values()) {
                    try {
                        getRecords(waitSyncLeaveRecords, processInstanceEnum, startTime, endTime, nextToken, maxResults, dingUserIds);
                    } catch (Exception e) {
                        log.error("同步请假记录异常", e);
                    }
                }
            }



        }
    }

    private void getRecords(List<AttendanceLeaveRecordEntity> waitSyncLeaveRecords, ProcessInstanceEnum processInstanceEnum, long startTime, long endTime, Long nextToken, long maxResults, List<String> dingUserIds) throws Exception {
        Rest<ListProcessInstanceIdsResponseBody.ListProcessInstanceIdsResponseBodyResult> rest = dingPersonService.getProcessInstanceIdByUserIds(processInstanceEnum.getValue(), startTime, endTime, nextToken, maxResults, dingUserIds);
        if (YesNoEnum.YES.getValue() == rest.getCode()) {
            if (ObjectUtils.isEmpty(rest.getData().getNextToken())) {
                List<String> processInstanceList = rest.getData().getList();
                for (String process : processInstanceList) {
                    AttendanceLeaveRecordEntity leaveRecord = new AttendanceLeaveRecordEntity();
                    leaveRecord.setRemark(process);
                    //  leaveRecord.setStaffId(item.getCustomerId());
                    waitSyncLeaveRecords.add(leaveRecord);
                    log.info("更新记录成功==>" + process);
                }
            } else {
                nextToken = DataUtil.converStr2Long(rest.getData().getNextToken(), 0L);
                getRecords(waitSyncLeaveRecords, processInstanceEnum, startTime, endTime, nextToken, maxResults, dingUserIds);
            }
        }
    }

    private void getRecords(List<AttendanceLeaveRecordEntity> waitSyncLeaveRecords, QiongLaiProcessInstanceEnum processInstanceEnum, long startTime, long endTime, Long nextToken, long maxResults, List<String> dingUserIds) throws Exception {
        Rest<ListProcessInstanceIdsResponseBody.ListProcessInstanceIdsResponseBodyResult> rest = dingPersonService.getProcessInstanceIdByUserIds(processInstanceEnum.getValue(), startTime, endTime, nextToken, maxResults, dingUserIds);
        if (YesNoEnum.YES.getValue() == rest.getCode()) {
            if (ObjectUtils.isEmpty(rest.getData().getNextToken())) {
                List<String> processInstanceList = rest.getData().getList();
                for (String process : processInstanceList) {
                    AttendanceLeaveRecordEntity leaveRecord = new AttendanceLeaveRecordEntity();
                    leaveRecord.setRemark(process);
                    //  leaveRecord.setStaffId(item.getCustomerId());
                    waitSyncLeaveRecords.add(leaveRecord);
                    log.info("更新记录成功==>" + process);
                }
            } else {
                nextToken = DataUtil.converStr2Long(rest.getData().getNextToken(), 0L);
                getRecords(waitSyncLeaveRecords, processInstanceEnum, startTime, endTime, nextToken, maxResults, dingUserIds);
            }
        }
    }

}